@@ -1,3 +1,5 @@
+/* -*- mode:C++; tab-width:8; c-basic-offset:8; indent-tabs-mode:true -*- */
+
 /*
  * written by J. Marcin Gorycki <marcin.gorycki@intel.com>
  *
@@ -26,13 +28,13 @@
 #include "scoreframe.h"
 
 #include <games-gconf.h>
+#include <games-frame.h>
+
 #include <gdk/gdkkeysyms.h>
 #include <config.h>
 #include <dirent.h>
 #include <string.h>
 
-GdkPixmap *pix;
-
 GdkPixbuf **pic;
 
 int LINES = 20;
@@ -57,6 +59,16 @@ bool rotateCounterClockWise = true;
 
 #define TETRIS_OBJECT "gnometris-tetris-object"
 
+#define KEY_OPTIONS_DIR "/apps/gnometris/options"
+#define KEY_BLOCK_PIXMAP "/apps/gnometris/options/block_pixmap"
+#define KEY_BACKGROUND_PIXMAP "/apps/gnometris/options/background_pixmap"
+#define KEY_STARTING_LEVEL "/apps/gnometris/options/starting_level"
+#define KEY_DO_PREVIEW "/apps/gnometris/options/do_preview"
+#define KEY_RANDOM_BLOCK_COLORS "/apps/gnometris/options/random_block_colors"
+#define KEY_ROTATE_COUNTER_CLOCKWISE "/apps/gnometris/options/rotate_counter_clock_wise"
+#define KEY_LINE_FILL_HEIGHT "/apps/gnometris/options/line_fill_height"
+#define KEY_LINE_FILL_PROBABILITY "/apps/gnometris/options/line_fill_probability"
+
 Tetris::Tetris(int cmdlLevel): 
 	blockPixmap(0),
 	bgPixmap(0),
@@ -122,11 +134,10 @@ Tetris::Tetris(int cmdlLevel):
 
 	/* init gconf */
 	gconf_client = gconf_client_get_default ();
-	if (!games_gconf_sanity_check_string (gconf_client, "/apps/gnometris/options/block_pixmap")) {
-		exit(1);
-	}
-	gconf_client_add_dir (gconf_client_get_default (), "/apps/gnometris/options", GCONF_CLIENT_PRELOAD_NONE, NULL);
-	gconf_client_notify_add (gconf_client_get_default (), "/apps/gnometris/options", gconfNotify, this, NULL, NULL);
+	games_gconf_sanity_check_string (gconf_client, KEY_BLOCK_PIXMAP);
+
+	gconf_client_add_dir (gconf_client, KEY_OPTIONS_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
+	gconf_client_notify_add (gconf_client, KEY_OPTIONS_DIR, gconfNotify, this, NULL, NULL);
 
 	initOptions ();
 
@@ -165,6 +176,7 @@ Tetris::Tetris(int cmdlLevel):
 	gtk_box_pack_end(GTK_BOX(vb2), scoreFrame->getWidget(), 0, 0, 0);
 
 	setOptions ();
+        setupScoreState ();
 
 	gtk_widget_show(hb);
 	gtk_widget_show(vb1);
@@ -222,6 +234,25 @@ Tetris::~Tetris()
 	delete[] pic;
 }
 
+void
+Tetris::setupScoreState ()
+{
+        gchar **names = NULL;
+        gfloat *scores = NULL;
+        time_t *scoretimes = NULL;
+	gint top = 0;
+
+	top = gnome_score_get_notable ("gnometris", NULL, &names, &scores, &scoretimes);
+	if (top > 0) {
+		gtk_widget_set_sensitive (gameMenuPtr[3].widget, TRUE);
+		g_strfreev (names);
+		g_free (scores);
+		g_free (scoretimes);
+	} else {
+		gtk_widget_set_sensitive (gameMenuPtr[3].widget, FALSE);
+	}
+}
+
 void 
 Tetris::setupdialogDestroy(GtkWidget *widget, void *d)
 {
@@ -242,12 +273,12 @@ Tetris::setupdialogResponse (GtkWidget *dialog, gint response_id, void *d)
 void
 Tetris::setupPixmap()
 {
-	char *pixname, *fullpixname;
+	gchar *pixname, *fullpixname;
         gchar * s;
 	
-	pixname = g_strdup_printf("gnometris/%s", blockPixmap);
+	pixname = g_build_filename ("gnometris", blockPixmap, NULL);
 	fullpixname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, pixname, FALSE, NULL);
-	g_free(pixname);
+	g_free (pixname);
 
 	if (!g_file_test (fullpixname, G_FILE_TEST_EXISTS))
 	{
@@ -256,6 +287,7 @@ Tetris::setupPixmap()
 						       GTK_MESSAGE_ERROR,
 						       GTK_BUTTONS_OK,
 						       _("Could not find the theme: \n%s\n\nPlease check your gnome-games installation"), fullpixname);
+		gtk_dialog_set_has_separator (GTK_DIALOG (w), FALSE);
 		gtk_dialog_run (GTK_DIALOG (w));
 		exit(1);
 	}
@@ -271,12 +303,12 @@ Tetris::setupPixmap()
 						       GTK_MESSAGE_ERROR,
 						       GTK_BUTTONS_OK,
 						       _("Can't load the image: \n%s\n\nPlease check your gnome-games installation"), fullpixname);
+		gtk_dialog_set_has_separator (GTK_DIALOG (w), FALSE);
 		gtk_dialog_run (GTK_DIALOG (w));
 		exit (1);
 	}
+	g_free (fullpixname);
 
-	gdk_pixbuf_render_pixmap_and_mask(image, &pix, NULL, 127);
-	
 	BLOCK_SIZE = gdk_pixbuf_get_height(image);
 	nr_of_colors = gdk_pixbuf_get_width(image) / BLOCK_SIZE;
 
@@ -292,15 +324,15 @@ Tetris::setupPixmap()
 
 	}
 
-	pixname = g_strdup_printf("gnometris/bg/%s", bgPixmap);
+	pixname = g_build_filename ("gnometris", "bg", bgPixmap, NULL);
 	fullpixname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, pixname, FALSE, NULL);
-	g_free(pixname);
+	g_free (pixname);
 
-	if(bgimage)
+	if (bgimage)
 		g_object_unref (G_OBJECT (bgimage));
 
 	if (g_file_test (fullpixname, G_FILE_TEST_EXISTS)) 
-		bgimage = gdk_pixbuf_new_from_file(fullpixname, NULL);
+		bgimage = gdk_pixbuf_new_from_file (fullpixname, NULL);
 	else {
                 /* If we can't find a .png look for a .jpg, this is
                  * especially pertinant since a lot of the original
@@ -308,24 +340,25 @@ Tetris::setupPixmap()
                 s = g_strrstr(fullpixname, ".png");
                 g_strlcpy(s, ".jpg", 5);
                 if (g_file_test (fullpixname, G_FILE_TEST_EXISTS)) {
-                        bgimage = gdk_pixbuf_new_from_file(fullpixname, NULL);
+                        bgimage = gdk_pixbuf_new_from_file (fullpixname, NULL);
                         s = g_strrstr (bgPixmap, ".png");
                         g_strlcpy (s, ".jpg", 5);
-                        gconf_client_set_string (gconf_client, "/apps/gnometris/options/background_pixmap", bgPixmap, NULL);
+                        gconf_client_set_string (gconf_client, KEY_BACKGROUND_PIXMAP, bgPixmap, NULL);
                 } else
                         bgimage = 0;
         }
+	g_free (fullpixname);
 
 	if (field)
 	{
-		field->updateSize(bgimage);
-		gtk_widget_queue_draw(field->getWidget());
+		field->updateSize (bgimage);
+		gtk_widget_queue_draw (field->getWidget());
 	}
 	
 	if (preview)
 	{
-		preview->updateSize();
-		gtk_widget_queue_draw(preview->getWidget());
+		preview->updateSize ();
+		gtk_widget_queue_draw (preview->getWidget());
 	}
 
 	// FIXME: this really sucks, but I can't find a better way to resize 
@@ -354,61 +387,67 @@ Tetris::initOptions ()
 	GError *error = NULL;
 
 	if (blockPixmap)
-		free (blockPixmap);
+		g_free (blockPixmap);
 
-	blockPixmap = gconf_client_get_string (gconf_client, "/apps/gnometris/options/block_pixmap", &error);
+	blockPixmap = gconf_client_get_string (gconf_client, KEY_BLOCK_PIXMAP, &error);
 	if (error) {
 		g_warning ("gconf error: %s\n", error->message);
 		g_error_free (error);
 		error = NULL;
 	}
+        if (blockPixmap == NULL)
+          blockPixmap = g_strdup ("7blocks-tig.png");
 
 	if (bgPixmap)
-		free (bgPixmap);
+		g_free (bgPixmap);
 
-	bgPixmap = gconf_client_get_string (gconf_client, "/apps/gnometris/options/background_pixmap", &error);
+	bgPixmap = gconf_client_get_string (gconf_client, KEY_BACKGROUND_PIXMAP, &error);
+	if (error) {
+		g_warning ("gconf error: %s\n", error->message);
+		g_error_free (error);
+		error = NULL;
+	}
+        if (bgPixmap == NULL)
+          bgPixmap = g_strdup ("gnometris-bg.jpg");
+
+	startingLevel = gconf_client_get_int (gconf_client, KEY_STARTING_LEVEL, &error);
+	if (error) {
+		g_warning ("gconf error: %s\n", error->message);
+		g_error_free (error);
+		error = NULL;
+	}
+        if (startingLevel < 1) 
+          startingLevel = 1;
+
+	do_preview = gconf_client_get_bool (gconf_client, KEY_DO_PREVIEW, &error);
 	if (error) {
 		g_warning ("gconf error: %s\n", error->message);
 		g_error_free (error);
 		error = NULL;
 	}
 
-	startingLevel = gconf_client_get_int (gconf_client, "/apps/gnometris/options/starting_level", &error);
-	if (error) {
-		g_warning ("gconf error: %s\n", error->message);
-		g_error_free (error);
-		error = NULL;
-	}
-
-	do_preview = gconf_client_get_bool (gconf_client, "/apps/gnometris/options/do_preview", &error);
-	if (error) {
-		g_warning ("gconf error: %s\n", error->message);
-		g_error_free (error);
-		error = NULL;
-	}
-
-	random_block_colors = gconf_client_get_bool (gconf_client, "/apps/gnometris/options/random_block_colors", &error);
+	random_block_colors = gconf_client_get_bool (gconf_client, KEY_RANDOM_BLOCK_COLORS, &error);
 	if (error) {
 		g_warning ("gconf error: %s\n", error->message);
 		g_error_free (error);
 		error = NULL;
 	}
 	
-	rotateCounterClockWise = gconf_client_get_bool (gconf_client, "/apps/gnometris/options/rotate_counter_clock_wise", &error);
+	rotateCounterClockWise = gconf_client_get_bool (gconf_client, KEY_ROTATE_COUNTER_CLOCKWISE, &error);
 	if (error) {
 		g_warning ("gconf error: %s\n", error->message);
 		g_error_free (error);
 		error = NULL;
 	}
 
-	line_fill_height = gconf_client_get_int (gconf_client, "/apps/gnometris/options/line_fill_height", &error);
+	line_fill_height = gconf_client_get_int (gconf_client, KEY_LINE_FILL_HEIGHT, &error);
 	if (error) {
 		g_warning ("gconf error: %s\n", error->message);
 		g_error_free (error);
 		error = NULL;
 	}
 
-	line_fill_prob = gconf_client_get_int (gconf_client, "/apps/gnometris/options/line_fill_probability", &error);
+	line_fill_prob = gconf_client_get_int (gconf_client, KEY_LINE_FILL_PROBABILITY, &error);
 	if (error) {
 		g_warning ("gconf error: %s\n", error->message);
 		g_error_free (error);
@@ -437,21 +476,24 @@ void
 Tetris::setSelectionPreview(GtkWidget *widget, void *d)
 {
 	Tetris *t = (Tetris*) d;
-	gconf_client_set_bool (t->gconf_client, "/apps/gnometris/options/do_preview", GTK_TOGGLE_BUTTON(widget)->active, NULL);
+	gconf_client_set_bool (t->gconf_client, KEY_DO_PREVIEW,
+			       GTK_TOGGLE_BUTTON (widget)->active, NULL);
 }
 
 void 
 Tetris::setSelectionBlocks(GtkWidget *widget, void *d)
 {
 	Tetris *t = (Tetris*) d;
-	gconf_client_set_bool (t->gconf_client, "/apps/gnometris/options/random_block_colors", GTK_TOGGLE_BUTTON(widget)->active, NULL);
+	gconf_client_set_bool (t->gconf_client, KEY_RANDOM_BLOCK_COLORS,
+			       GTK_TOGGLE_BUTTON (widget)->active, NULL);
 }
 
 void 
 Tetris::setRotateCounterClockWise(GtkWidget *widget, void *d)
 {
 	Tetris *t = (Tetris*) d;
-	gconf_client_set_bool (t->gconf_client, "/apps/gnometris/options/rotate_counter_clock_wise", GTK_TOGGLE_BUTTON(widget)->active, NULL);
+	gconf_client_set_bool (t->gconf_client, KEY_ROTATE_COUNTER_CLOCKWISE,
+			       GTK_TOGGLE_BUTTON (widget)->active, NULL);
 }
 
 void
@@ -460,7 +502,8 @@ Tetris::setSelection(GtkWidget *widget, void *data)
 	Tetris *t;
 
       	t = (Tetris *)g_object_get_data (G_OBJECT (widget), TETRIS_OBJECT);
-	gconf_client_set_string (t->gconf_client, "/apps/gnometris/options/block_pixmap", (char *)data, NULL);
+	gconf_client_set_string (t->gconf_client, KEY_BLOCK_PIXMAP,
+				 (char *)data, NULL);
 }
 
 void
@@ -469,7 +512,8 @@ Tetris::setBGSelection(GtkWidget *widget, void *data)
 	Tetris *t;
        
 	t = (Tetris *)g_object_get_data (G_OBJECT (widget), TETRIS_OBJECT);
-	gconf_client_set_string (t->gconf_client, "/apps/gnometris/options/background_pixmap", (char *)data, NULL);
+	gconf_client_set_string (t->gconf_client, KEY_BACKGROUND_PIXMAP,
+				 (char *)data, NULL);
 }
 
 void
@@ -483,7 +527,8 @@ Tetris::lineFillHeightChanged (GtkWidget *spin, gpointer data)
 {
 	Tetris *t = (Tetris *)data;
 	gint value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin));
-	gconf_client_set_int (t->gconf_client, "/apps/gnometris/options/line_fill_height", value, NULL);
+	gconf_client_set_int (t->gconf_client, KEY_LINE_FILL_HEIGHT,
+			      value, NULL);
 }
 
 void
@@ -491,7 +536,8 @@ Tetris::lineFillProbChanged (GtkWidget *spin, gpointer data)
 {
 	Tetris *t = (Tetris *)data;
 	gint value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin));
-	gconf_client_set_int (t->gconf_client, "/apps/gnometris/options/line_fill_probability", value, NULL);
+	gconf_client_set_int (t->gconf_client, KEY_LINE_FILL_PROBABILITY,
+			      value, NULL);
 }
 
 void
@@ -499,7 +545,8 @@ Tetris::startingLevelChanged (GtkWidget *spin, gpointer data)
 {
 	Tetris *t = (Tetris *)data;
 	gint value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin));
-	gconf_client_set_int (t->gconf_client, "/apps/gnometris/options/starting_level", value, NULL);
+	gconf_client_set_int (t->gconf_client, KEY_STARTING_LEVEL,
+			      value, NULL);
 }
 
 void
@@ -564,6 +611,8 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 	GtkWidget *box, *box2;
 	GtkWidget *label;
 	GtkWidget *frame;
+	GtkWidget *table;
+	GtkWidget *hbox, *fvbox, *space_label;
 	GtkObject *adj;
         
 	Tetris *t = (Tetris*) d;
@@ -580,160 +629,144 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 					    (GtkDialogFlags)0,
 					    GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 					    NULL);
-
-	gtk_container_set_border_width(GTK_CONTAINER(t->setupdialog), 10);
+	gtk_dialog_set_has_separator (GTK_DIALOG (t->setupdialog), FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (t->setupdialog), 12);
 	g_signal_connect (t->setupdialog, "close",
 			  G_CALLBACK (setupdialogDestroy), d);
 	g_signal_connect (t->setupdialog, "response",
 			  G_CALLBACK (setupdialogResponse), d);
 
-	allBoxes = gtk_vbox_new(FALSE, 5);
-	
-	frame = gtk_vbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(allBoxes), frame, TRUE, TRUE, 0);
-	
-	box = gtk_vbox_new(FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(frame), box);
-
-	gtk_widget_show(box);
-	gtk_widget_show(frame);
-
-	box2 = gtk_hbox_new(FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(box2), 10);
-	gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
+	frame = games_frame_new (_("Setup"));
+	table = gtk_table_new (3, 2, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 0);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
 
 	/* pre-filled rows */
 	label = gtk_label_new(_("Number of pre-filled rows:"));
-	gtk_box_pack_start(GTK_BOX(box2), label, 0, 0, 0);
-	gtk_widget_show(label);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
 
-	adj = gtk_adjustment_new(t->line_fill_height,0,LINES-1,1,5,0);
-	t->fill_height_spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adj),10,0);
+	adj = gtk_adjustment_new (t->line_fill_height, 0, LINES-1, 1, 5, 0);
+	t->fill_height_spinner = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 10, 0);
 	gtk_spin_button_set_update_policy
-		(GTK_SPIN_BUTTON(t->fill_height_spinner), GTK_UPDATE_ALWAYS);
+		(GTK_SPIN_BUTTON (t->fill_height_spinner), GTK_UPDATE_ALWAYS);
 	gtk_spin_button_set_snap_to_ticks 
-		(GTK_SPIN_BUTTON(t->fill_height_spinner), TRUE);
+		(GTK_SPIN_BUTTON (t->fill_height_spinner), TRUE);
 	g_signal_connect (t->fill_height_spinner, "value_changed",
 			  G_CALLBACK (lineFillHeightChanged), t);
-	gtk_box_pack_end(GTK_BOX(box2), t->fill_height_spinner, 0, 0, 0);
-	gtk_widget_show(t->fill_height_spinner);	
-	gtk_widget_show(box2);
+	gtk_table_attach_defaults (GTK_TABLE (table), t->fill_height_spinner, 1, 2, 0, 1);
 
 	/* pre-filled rows density */
-	box2 = gtk_hbox_new(FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(box2), 10);
-	gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
+	label = gtk_label_new (_("Density of blocks in a pre-filled row:"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
 
-	label = gtk_label_new(_("Density of blocks in a pre-filled row:"));
-	gtk_box_pack_start(GTK_BOX(box2), label, 0, 0, 0);
-	gtk_widget_show(label);
-
-	adj = gtk_adjustment_new(t->line_fill_prob,0,10,1,5,0);
-	t->fill_prob_spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adj),10,0);
-	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(t->fill_prob_spinner),
+	adj = gtk_adjustment_new (t->line_fill_prob, 0, 10, 1, 5, 0);
+	t->fill_prob_spinner = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 10, 0);
+	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (t->fill_prob_spinner),
 					  GTK_UPDATE_ALWAYS);
 	gtk_spin_button_set_snap_to_ticks
-		(GTK_SPIN_BUTTON(t->fill_prob_spinner), TRUE);
+		(GTK_SPIN_BUTTON (t->fill_prob_spinner), TRUE);
 	g_signal_connect (t->fill_prob_spinner, "value_changed",
 		          G_CALLBACK (lineFillProbChanged), t);
-	gtk_box_pack_end(GTK_BOX(box2), t->fill_prob_spinner, 0, 0, 0);
-	gtk_widget_show(t->fill_prob_spinner);	
-	gtk_widget_show(box2);
+	gtk_table_attach_defaults (GTK_TABLE (table), t->fill_prob_spinner, 1, 2, 1, 2);
 
 	/* starting level */
-	box2 = gtk_hbox_new(FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(box2), 10);
-	gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
+	label = gtk_label_new (_("Starting Level:"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2, 3);
 
-	label = gtk_label_new(_("Starting Level:"));
-	gtk_box_pack_start(GTK_BOX(box2), label, 0, 0, 0);
-	gtk_widget_show(label);
-
-	adj = gtk_adjustment_new(t->startingLevel, 1, 10, 1, 5, 10);
-	t->sentry = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 10, 0);
-	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(t->sentry),
-					  GTK_UPDATE_ALWAYS);
-	gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(t->sentry), TRUE);
+	adj = gtk_adjustment_new (t->startingLevel, 1, 10, 1, 5, 10);
+	t->sentry = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 10, 0);
+	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (t->sentry),
+					   GTK_UPDATE_ALWAYS);
+	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (t->sentry), TRUE);
 	g_signal_connect (t->sentry, "value_changed",
 			  G_CALLBACK (startingLevelChanged), t);
-	gtk_box_pack_end(GTK_BOX(box2), t->sentry, FALSE, 0, 0);
-	gtk_widget_show(t->sentry);
-	gtk_widget_show(box2);
+	gtk_table_attach_defaults (GTK_TABLE (table), t->sentry, 1, 2, 2, 3);
+
+	gtk_container_add (GTK_CONTAINER (frame), table);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (t->setupdialog)->vbox), frame, 
+			    FALSE, FALSE, 0);
+
+	frame = games_frame_new (_("Operation"));
+	fvbox = gtk_vbox_new (FALSE, FALSE);
+	gtk_box_set_spacing (GTK_BOX (fvbox), 6);
 
 	/* preview next block */
 	t->do_preview_toggle =
-		gtk_check_button_new_with_label(_("Preview next block"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t->do_preview_toggle),
+		gtk_check_button_new_with_label (_("Preview next block"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (t->do_preview_toggle),
 				     do_preview);
 	g_signal_connect (t->do_preview_toggle, "clicked",
 			  G_CALLBACK (setSelectionPreview), d);
-	gtk_box_pack_start(GTK_BOX(box), t->do_preview_toggle, 0, 0, 0);
-	gtk_widget_show(t->do_preview_toggle);
+	gtk_box_pack_start (GTK_BOX (fvbox), t->do_preview_toggle, 0, 0, 0);
 
 	/* random blocks */
 	t->random_block_colors_toggle =
-		gtk_check_button_new_with_label(_("Random block colors"));
+		gtk_check_button_new_with_label(_("Use random block colors"));
 	gtk_toggle_button_set_active
-		(GTK_TOGGLE_BUTTON(t->random_block_colors_toggle),
+		(GTK_TOGGLE_BUTTON (t->random_block_colors_toggle),
 		 random_block_colors);
 	g_signal_connect (t->random_block_colors_toggle, "clicked",
 			  G_CALLBACK (setSelectionBlocks), d);
-	gtk_box_pack_start(GTK_BOX(box), t->random_block_colors_toggle,
-			   0, 0, 0);
-	gtk_widget_show(t->random_block_colors_toggle);
+	gtk_box_pack_start (GTK_BOX (fvbox), t->random_block_colors_toggle,
+			    0, 0, 0);
 
 	/* rotate counter clock wise */
  	t->rotate_counter_clock_wise_toggle =
-		gtk_check_button_new_with_label(_("Rotate counterclockwise"));
+		gtk_check_button_new_with_label (_("Rotate blocks counterclockwise"));
  	gtk_toggle_button_set_active
-		(GTK_TOGGLE_BUTTON(t->rotate_counter_clock_wise_toggle),
+		(GTK_TOGGLE_BUTTON (t->rotate_counter_clock_wise_toggle),
 		 rotateCounterClockWise);
 	g_signal_connect (t->rotate_counter_clock_wise_toggle, "clicked",
 			  G_CALLBACK (setRotateCounterClockWise), d);
- 	gtk_box_pack_start(GTK_BOX(box), t->rotate_counter_clock_wise_toggle,
-			   0, 0, 0);
- 	gtk_widget_show(t->rotate_counter_clock_wise_toggle);
+ 	gtk_box_pack_start (GTK_BOX (fvbox), t->rotate_counter_clock_wise_toggle,
+			    0, 0, 0);
 
-	gtk_widget_show(box);
+	gtk_container_add (GTK_CONTAINER (frame), fvbox);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (t->setupdialog)->vbox), frame, 
+			    FALSE, FALSE, 0);
+
+	frame = games_frame_new (_("Theme"));
+	table = gtk_table_new (2, 2, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 0);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
 
 	/* Block pixmap */
-	box2 = gtk_hbox_new(FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(box2), 10);
-	gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
-	label = gtk_label_new(_("Block Pixmap: "));
-	gtk_box_pack_start(GTK_BOX(box2), label, 0, 0, 0);
-	gtk_widget_show(label);
+	label = gtk_label_new(_("Block image:"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
 
-	GtkWidget *omenu = gtk_option_menu_new();
-	GtkWidget *menu = gtk_menu_new();
-	t->fillMenu(menu, t->blockPixmap, "gnometris",
-	            (GtkSignalFunc)setSelection);
-	gtk_widget_show(omenu);
-	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);
-	gtk_box_pack_end(GTK_BOX(box2), omenu, 0, 0, 0);
-	gtk_widget_show(box2);
+	GtkWidget *omenu = gtk_option_menu_new ();
+	GtkWidget *menu = gtk_menu_new ();
+	t->fillMenu (menu, t->blockPixmap, "gnometris",
+		     (GtkSignalFunc)setSelection);
+
+	gtk_option_menu_set_menu(GTK_OPTION_MENU (omenu), menu);
+	gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 0, 1);
 
 	/* background pixmap */
-	box2 = gtk_hbox_new(FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(box2), 10);
-	gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
-	label = gtk_label_new(_("Background Pixmap: "));
-	gtk_box_pack_start(GTK_BOX(box2), label, 0, 0, 0);
-	gtk_widget_show(label);
-	omenu = gtk_option_menu_new();
-	menu = gtk_menu_new();
-	t->fillMenu(menu, t->bgPixmap, "gnometris/bg",
-		    (GtkSignalFunc)setBGSelection, true);
-	gtk_widget_show(omenu);
-	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);
-	gtk_box_pack_end(GTK_BOX(box2), omenu, 0, 0, 0);
-	gtk_widget_show(box2);
+	label = gtk_label_new(_("Background image:"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
 
-	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(t->setupdialog)->vbox),
-			            allBoxes);
-	gtk_widget_show(allBoxes);
+	omenu = gtk_option_menu_new ();
+	menu = gtk_menu_new ();
+	gchar *tmp = g_build_filename ("gnometris", "bg", NULL);
+	t->fillMenu (menu, t->bgPixmap, tmp,
+		     (GtkSignalFunc)setBGSelection, true);
+	g_free (tmp);
+	gtk_option_menu_set_menu(GTK_OPTION_MENU (omenu), menu);
+	gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 1, 2);
+
+	gtk_container_add (GTK_CONTAINER (frame), table);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (t->setupdialog)->vbox), frame, 
+			    FALSE, FALSE, 0);
 	
-	gtk_widget_show(t->setupdialog);
+	gtk_widget_show_all (t->setupdialog);
 	return TRUE;
 }
 
@@ -971,6 +1004,7 @@ Tetris::endOfGame()
 	color_next = -1;
 	blocknr_next = -1;
 	rot_next = -1;
+        gnome_canvas_item_hide (pauseMessage);
         gnome_canvas_item_show (gameoverMessage);
         gnome_canvas_item_raise_to_top (gameoverMessage);
 
